// color.cc	v1.3	22. December 2004	by eddy

/*  lego -- this program generates images of LEGO(R)-like models.
          
                Copyright (C) 1998 - 2008 Edmund Nussbaum.
          
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
          
    This program is distributed in the hope that you will have a lot of
    fun when using it, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
    the GNU General Public License for more details.
          
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Please report your experiences, suggestions or whatever related
    to this program to:

    sibylle.eddy @ t-online.de
    en @ mpifr-bonn.mpg.de

    or send mail to:
    Edmund Nussbaum, Richard-Wagner-Str. 59, 53115 Bonn, Germany
*/        

#include "color.h"

color::color(void)
	{
	flagtran = flagdomi = false;
	code = 0;
	}

color::color(String str)
	{
	String cstr(upcase(str));
	if(     cstr.contains("BLACK"      )) code = C_BLACK;
	else if(cstr.contains("SCHWARZ"    )) code = C_BLACK;
	else if(cstr.contains("RED"        )) code = C_RED;
	else if(cstr.contains("ROT"        )) code = C_RED;
	else if(cstr.contains("PINK"       )) code = C_PINK;
	else if(cstr.contains("ROSA"       )) code = C_PINK;
	else if(cstr.contains("YELLOW"     )) code = C_YELLOW;
	else if(cstr.contains("GELB"       )) code = C_YELLOW;
	else if(cstr.contains("LIGHTGREEN" )) code = C_LIGHTGREEN;
	else if(cstr.contains("HELLGRUEN"  )) code = C_LIGHTGREEN;
	else if(cstr.contains("GREEN"      )) code = C_GREEN;
	else if(cstr.contains("GRUEN"      )) code = C_GREEN;
	else if(cstr.contains("LIGHTBLUE"  )) code = C_LIGHTBLUE;
	else if(cstr.contains("HELLBLAU"   )) code = C_LIGHTBLUE;
	else if(cstr.contains("BLUE"       )) code = C_BLUE;
	else if(cstr.contains("BLAU"       )) code = C_BLUE;
	else if(cstr.contains("DARKGREY"   )) code = C_DARKGREY;
	else if(cstr.contains("DUNKELGRAU" )) code = C_DARKGREY;
	else if(cstr.contains("GREY"       )) code = C_GREY;
	else if(cstr.contains("GRAU"       )) code = C_GREY;
	else if(cstr.contains("WHITE"      )) code = C_WHITE;
	else if(cstr.contains("WEISS"      )) code = C_WHITE;
	else if(cstr.contains("MAGENTA"    )) code = C_MAGENTA;
	else if(cstr.contains("BROWN"      )) code = C_BROWN;
	else if(cstr.contains("BRAUN"      )) code = C_BROWN;
	else if(cstr.contains("BEIGE"      )) code = C_BEIGE;
	else if(cstr.contains("ORANGE"     )) code = C_ORANGE;
	else if(cstr.contains("LILAC"      )) code = C_LILAC;
	else if(cstr.contains("LILA"       )) code = C_LILAC;
	else if(cstr.contains("TRANSPARENT")) code = C_WHITE;
	else code = 0;
	flagtran = flagdomi = false;
	if(cstr.contains("TRANSPARENT")) flagtran = true;
	}

color::color(int idx,bool fdomi /* =false */)
	{
	code = idx;
	flagdomi = fdomi;
	flagtran = false;
	}

color color::dark(void)
	{
	static const int MASK = 1 << COLBITS;
	color darkcolor(*this);
	darkcolor.code |= MASK;
	return darkcolor;
	}

void color::settransparent(void)
	{
	flagtran = true;
	}

void color::setdominant(void)
	{
	flagdomi = true;
	}

bool color::transparent(void) const
	{
	return flagtran;
	}

bool color::dominant(void) const
	{
	return flagdomi;
	}

bool color::good(void) const
	{
	static const int MASK = (-1 << COLBITS) ^ -1;
	if(MASK & code)	return true;
	return false;
	}

double color::tranfac(void) const
	{
	if(flagtran) return TRANSFACTOR;
	return 1.0;
	}

///////////////////////////////////////////////////////////////////////////

static unsigned char R[COLORS];
static unsigned char G[COLORS];
static unsigned char B[COLORS];

void initrgbcolors(void)
	{
	for(int i=0;i<COLORS;i++) R[i] = G[i] = B[i] = 0;
	R[C_BLACK     ]=  80;  G[C_BLACK     ]=  80;  B[C_BLACK     ]=  80;
	R[C_RED       ]= 255;  G[C_RED       ]=  60;  B[C_RED       ]=  50;
	R[C_PINK      ]= 255;  G[C_PINK      ]= 192;  B[C_PINK      ]= 203;
	R[C_YELLOW    ]= 250;  G[C_YELLOW    ]= 230;  B[C_YELLOW    ]=   0;
	R[C_GREEN     ]=  30;  G[C_GREEN     ]= 200;  B[C_GREEN     ]=  50;
	R[C_LIGHTGREEN]= 144;  G[C_LIGHTGREEN]= 238;  B[C_LIGHTGREEN]= 144;
	R[C_BLUE      ]= 100;  G[C_BLUE      ]= 120;  B[C_BLUE      ]= 255;
	R[C_LIGHTBLUE ]= 180;  G[C_LIGHTBLUE ]= 200;  B[C_LIGHTBLUE ]= 255;
	R[C_GREY      ]= 200;  G[C_GREY      ]= 200;  B[C_GREY      ]= 200;
	R[C_DARKGREY  ]= 130;  G[C_DARKGREY  ]= 130;  B[C_DARKGREY  ]= 130;
	R[C_WHITE     ]= 250;  G[C_WHITE     ]= 250;  B[C_WHITE     ]= 250;
	R[C_MAGENTA   ]= 240;  G[C_MAGENTA   ]=   0;  B[C_MAGENTA   ]= 240;
	R[C_BROWN     ]= 130;  G[C_BROWN     ]=  50;  B[C_BROWN     ]=  40;
	R[C_BEIGE     ]= 230;  G[C_BEIGE     ]= 190;  B[C_BEIGE     ]= 140;
	R[C_ORANGE    ]= 248;  G[C_ORANGE    ]= 164;  B[C_ORANGE    ]=   0;
	R[C_LILAC     ]= 153;  G[C_LILAC     ]=  50;  B[C_LILAC     ]= 204;
	R[C_BACKGROUND]= 255;  G[C_BACKGROUND]= 255;  B[C_BACKGROUND]= 255;
	R[C_OUTLINE   ]=   0;  G[C_OUTLINE   ]=   0;  B[C_OUTLINE   ]=   0;
	}

rgbcolor::rgbcolor(void)
	{
	rval = gval = bval = 0;
	}

rgbcolor::rgbcolor(int val)
	{
	static const int MASK = (-1 << COLBITS) ^ -1;
	static const int DARK =   1 << COLBITS;
	int idx = val & MASK;	
	rval = R[idx];
	gval = G[idx];
	bval = B[idx];
	if(val & DARK)
		{
		rval >>= 1;
		gval >>= 1;
		bval >>= 1;
//		rval = (unsigned char)(0.7*rval);
//		gval = (unsigned char)(0.7*gval);
//		bval = (unsigned char)(0.7*bval);
		}
	}

//
