// color.h	v1.3	1. October 2006	by eddy

/*  lego -- this program generates images of LEGO(R)-like models.

    		Copyright (C) 1998 - 2008 Edmund Nussbaum.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that you will have a lot of 
    fun when using it, but WITHOUT ANY WARRANTY; without even the implied 
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
    the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Please report your experiences, suggestions or whatever related
    to this program to: 

    sibylle.eddy @ t-online.de
    en @ mpifr-bonn.mpg.de 

    or send mail to:
    Edmund Nussbaum, Richard-Wagner-Str. 59, 53115 Bonn, Germany 
*/

#ifndef _COLOR_H
#define _COLOR_H

#include "String.h"

const int C_BADCOLOR    =  0;	// never
const int C_BLACK       =  1;	// define
const int C_RED         =  2;	// more
const int C_PINK        =  3;	// than
const int C_YELLOW      =  4;	// 2^COLBITS
const int C_GREEN       =  5;	// colors.
const int C_LIGHTGREEN  =  6;
const int C_BLUE        =  7;
const int C_LIGHTBLUE	=  8;
const int C_GREY        =  9;
const int C_DARKGREY	= 10;
const int C_WHITE       = 11;
const int C_MAGENTA		= 12;
const int C_BROWN		= 13;
const int C_BEIGE		= 14;
const int C_ORANGE		= 15;
const int C_LILAC		= 16;
const int C_BACKGROUND	= 17;
const int C_OUTLINE		= 18;
const int COLORS		= 19;	// No. of colors.

static const int COLBITS = 5;	// There can be 2^COLBITS different colors. 
static const int COLFLAG = 1;	// Needed for flagdark.
static const int COLMASK = -1 << (COLBITS+COLFLAG); // Mask to cut off code space.

class color
	{
	private:
	static const double TRANSFACTOR = 0.41;
//	static const double TRANSFACTOR = 0.45;
	bool flagtran;	// Indicates if color is transparent.
	bool flagdomi;	// Indicates if color is dominant.
	int code;		// pixeldata will be (pixeldata & COLMASK) | code.
	public:
	color(void);
	color(String);					// Construct color from keyword.
	color(int idx,bool fdomi=false);// Construct color from index.
	~color(void) { }
	color dark(void);				// Return color to half brightness.
	void settransparent(void);		// Set color transparent.
	void setdominant(void);			// Set color dominant.
	bool transparent(void) const;	// Return true if color is  transparent.
	bool dominant(void) const;		// Return true if color is dominant.
	bool good(void) const;			// Return true if color is valid.
	int encode(int e) const {return(e&COLMASK)|code;} // Insert color code.
	double tranfac(void) const;		// Return TRANSFACTOR when transparent.
	};
	
//////////////////////////////////////////////////////////////////////////////
	
void initrgbcolors(void);

class rgbcolor
	{
	private:
	unsigned char rval,gval,bval;		// Red, Green, Blue.
	public:
	rgbcolor(void);
	rgbcolor(int);						// Construct from pixel code.
	~rgbcolor(void) { }
	char r(void) const { return rval; }	// Return value for red.
	char g(void) const { return gval; }	// Return value for green.
	char b(void) const { return bval; }	// Return value for blue.
	};	

//////////////////////////////////////////////////////////////////////////////
	
// Predefined colors:

const color OUTLINE(C_OUTLINE,true);
const color BACKGROUND(C_BACKGROUND);
const color RED(C_RED);
const color GREY(C_GREY);
const color BLUE(C_BLUE);
const color YELLOW(C_YELLOW);
const color GREEN(C_GREEN);
const color BLACK(C_BLACK);
const color WHITE(C_WHITE);
const color TRANSPARENT("TRANSPARENT");

#endif //  _COLOR_H

//
