// group.cc	v1.3	9. June 2007	by eddy

/*  lego -- this program generates images of LEGO(R)-like models.

    		Copyright (C) 1998 - 2008 Edmund Nussbaum.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that you will have a lot of 
    fun when using it, but WITHOUT ANY WARRANTY; without even the implied 
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
    the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Please report your experiences, suggestions or whatever related
    to this program to: 
    
    sibylle.eddy @ t-online.de                   
    en @ mpifr-bonn.mpg.de 

    or send mail to:
    Edmund Nussbaum, Richard-Wagner-Str. 59, 53115 Bonn, Germany 
*/

#include "group.h"
#include "param.h"
#include <stdlib.h>

group::~group(void)
	{
	if(grp_member==NULL) return;
	int i=0;
	while(grp_member[i]!=NULL) delete grp_member[i++];
	delete [] grp_member;
	}

void group::init(param& p,int members)
	{
	sha_org = p.org();
	sha_siz = p.siz();
	sha_col = p.col();
	sha_rot = p.rot();
	grp_member = new shape*[members+1];
	for(int i=0;i<=members;i++) grp_member[i] = NULL;
	}

void group::draw(room& r)
	{
	int i=0;
	if(grp_member==NULL) return;
	while(grp_member[i]!=NULL) grp_member[i++]->draw(r);
	}

//////////////////////////////////////////////////////////////////////////////

static vector circularXY(double phi)
	{
	phi *= DEGREE;
	return vector( cos(phi), sin(phi), 0 );
	}

static vector circularXZ(double phi)
	{
	phi *= DEGREE;
	return vector( sin(phi), 0, 2.5*cos(phi) );
	}

//////////////////////////////////////////////////////////////////////////////
// GROUPES of shapes:
/////////////////////

brick::brick(param& p)
	{
	int i = 0;
	int members = 1 + (int)p.siz().x() * (int)p.siz().y();
	init(p,members);
	vector vo(0.5,0.5,sha_siz.z());
	vector vx(1,0,0);
	vector vy(0,1,0);
	int xlimit = (int)sha_siz.x();
	int ylimit = (int)sha_siz.y();
	for(int x=0;x<xlimit;x++)
		{
		vector vxx(vx*x+vo);
		for(int y=0;y<ylimit;y++)
			{
			vector rel(vy*y+vxx);
			grp_member[i++] = new stud1(rel,p);
			}
		}
	grp_member[i++] = new body(p);
//	grp_member[i] = NULL;
	}

ground::ground(param& p)
	{
	int i = 0;
	int members = 1 + (int)p.siz().x() * (int)p.siz().y();
	init(p,members);
	vector vo(0.5,0.5,0);
	vector vx(1,0,0);
	vector vy(0,1,0);
	int xlimit = (int)sha_siz.x();
	int ylimit = (int)sha_siz.y();
	for(int x=0;x<xlimit;x++)
		{
		vector vxx(vx*x+vo);
		for(int y=0;y<ylimit;y++)
			{
			vector rel(vy*y+vxx);
			grp_member[i++] = new stud1(rel,p);
			}
		}
	grp_member[i++] = new groundbody(p);
//	grp_member[i] = NULL;
	}

antiroof::antiroof(param& p)
	{
	int i = 0;
	int members = 1 + (int)p.siz().x() * (int)p.siz().y();
	init(p,members);
	vector vo(0.5,0.5,sha_siz.z());
	vector vx(1,0,0);
	vector vy(0,1,0);
	int xlimit = (int)sha_siz.x();
	int ylimit = (int)sha_siz.y();
	for(int x=0;x<xlimit;x++)
		{
		vector vxx(vx*x+vo);
		grp_member[i++] = new stud1(vxx,p);
		for(int y=1;y<ylimit;y++)
			{
			vector rel(vy*y+vxx);
			grp_member[i++] = new stud2(rel,p);
			}
		}
	grp_member[i++] = new antiroofbody(p);
//	grp_member[i] = NULL;
	}

roof::roof(param& p)
	{
	int i = 0;
	int members = 1 + (int)p.siz().x();
	init(p,members);
	vector vo(0.5,0.5,sha_siz.z());
	vector vx(1,0,0);
	int xlimit = (int)sha_siz.x();
	for(int x=0;x<xlimit;x++)
		{
		vector rel(vx*x+vo);
		grp_member[i++] = new stud1(rel,p);
		}
	grp_member[i++] = new roofbody(p);
//	grp_member[i] = NULL;
	}

roofcorner::roofcorner(param& p)
	{
	int i = 0;
	init(p,2);
	vector vo(0.5,0.5,sha_siz.z());
	grp_member[i++] = new stud1(vo,p);
	grp_member[i++] = new roofcornerbody(p);
//	grp_member[i] = NULL;
	}

corner::corner(param& p)
	{
	int i = 0;
	init(p,4);
	vector vo0(0.5,0.5,sha_siz.z());
	vector vo1(vo0+vector(1,0,0));
	vector vo2(vo0+vector(0,1,0));
	grp_member[i++] = new stud1(vo0,p);
	grp_member[i++] = new stud1(vo1,p);
	grp_member[i++] = new stud1(vo2,p);
	grp_member[i++] = new cornerbody(p);
//	grp_member[i] = NULL;
	}

round1::round1(param& p)
	{
	int i = 0;
	init(p,2);
	vector vo(0.5,0.5,sha_siz.z());
	if(sha_siz.z()>1) grp_member[i++] = new stud2(vo,p);
	else grp_member[i++] = new stud1(vo,p);
	grp_member[i++] = new roundbody(p);
//	grp_member[i] = NULL;
	}

curved1::curved1(param& p)
	{
	int i = 0;
	int members = 1 + (int)p.siz().x() * (int)( p.siz().y() - 1 );
	init(p,members);
	vector vo(0.5,0.5,sha_siz.z());
	vector vx(1,0,0);
	vector vy(0,1,0);
	int xlimit = (int)sha_siz.x();
	int ylimit = (int)sha_siz.y()-1;
	for(int y=0;y<ylimit;y++)
		{
		vector vyy(vy*y+vo);
		for(int x=0;x<xlimit;x++)
			{
			vector rel(vx*x+vyy);
			grp_member[i++] = new stud1(rel,p);
			}
		}
	grp_member[i++] = new curved1body(p);
//	grp_member[i] = NULL;
	}

curved2::curved2(param& p)
	{
	int i = 0;
	int members = 1 + (int)p.siz().x() * (int)( p.siz().y() - 2 );
	init(p,members);
	vector vo(0.5,0.5,sha_siz.z());
	vector vx(1,0,0);
	vector vy(0,1,0);
	int xlimit = (int)sha_siz.x();
	int ylimit = (int)sha_siz.y();
	for(int y=1;y<ylimit-1;y++)
		{
		vector vyy(vy*y+vo);
		for(int x=0;x<xlimit;x++)
			{
			vector rel(vx*x+vyy);
			grp_member[i++] = new stud1(rel,p);
			}
		}
	grp_member[i++] = new curved2body(p);
//	grp_member[i] = NULL;
	}

curved5::curved5(param& p)
	{
	int i = 0;
	int members = 1 + (int)( p.siz().y() );
	init(p,members);
	
	vector vo(0.5*sha_siz.x(),0.5,sha_siz.z());
	int ylimit = (int)sha_siz.y();
	for(int y=0;y<ylimit;y++)
		{
		vector rel(vo+y*YUNIT);
		grp_member[i++] = new stud2(rel,p);
		}
	grp_member[i++] = new curved4(p);
//	grp_member[i] = NULL;
	}

roundcorner1::roundcorner1(param& p)
	{
	int i = 0;
	init(p,3);
	vector rel(1.5,0.5,sha_siz.z());
	grp_member[i++] = new stud1(rel,p);
	rel = vector(0.5,1.5,sha_siz.z());
	grp_member[i++] = new stud1(rel,p);
	grp_member[i++] = new roundcorner1body(p);
//	grp_member[i] = NULL;
	}

roundcorner2::roundcorner2(param& p)
	{
	int i = 0;
	init(p,12);
	vector spos(0.5,0.5,sha_siz.z());
	vector sdel(1,0,0);
	grp_member[i++] = new stud1(spos,p);
	spos += sdel;
	grp_member[i++] = new stud1(spos,p);
	spos += sdel;
	grp_member[i++] = new stud1(spos,p);
	spos += sdel;
	grp_member[i++] = new stud1(spos,p);
	spos = vector(0.5,1.5,sha_siz.z());
	grp_member[i++] = new stud1(spos,p);
	spos += sdel;
	grp_member[i++] = new stud1(spos,p);
	spos += sdel;
	grp_member[i++] = new stud1(spos,p);
	spos = vector(0.5,2.5,sha_siz.z());
	grp_member[i++] = new stud1(spos,p);
	spos += sdel;
	grp_member[i++] = new stud1(spos,p);
	spos += sdel;
	grp_member[i++] = new stud1(spos,p);
	spos = vector(0.5,3.5,sha_siz.z());
	grp_member[i++] = new stud1(spos,p);
	grp_member[i++] = new roundcorner2body(p);
//	grp_member[i] = NULL;
	}

axisvertical::axisvertical(param& p)
	{
	int i = 0;
	init(p,2);
	vector norg = sha_org;
	norg += vector(0.5*sha_siz.x(),0.5*sha_siz.y(),sha_siz.z());
	vector ndir(0,0,2.5);
	param navepar(norg,ndir,sha_col,sha_rot);
	grp_member[i++] = new navepin1(navepar);
	grp_member[i++] = new body(p);
//	grp_member[i] = NULL;
	}

axisone::axisone(param& p)
	{
	int i = 0;
	init(p,2);
	vector norg = sha_org+vector(0.5*sha_siz.x(),sha_siz.y(),2);
	vector ndir(0,1,0);
	param navepar(norg,ndir,sha_col,sha_rot);
	grp_member[i++] = new navepin1(navepar);
	grp_member[i++] = new brick(p);
//	grp_member[i] = NULL;
	}

axistwo::axistwo(param& p)
	{
	int i = 0;
	init(p,3);
	vector norg = sha_org+vector(0.5*sha_siz.x(),0,2);
	vector ndir(0,-1,0);
	param navepar(norg,ndir,sha_col,sha_rot);
	grp_member[i++] = new navepin1(navepar);
	norg += vector(0,sha_siz.y(),0);
	ndir = vector(0,1,0);
	navepar.setorg(norg);
	navepar.setsiz(ndir);
	grp_member[i++] = new navepin1(navepar);
	grp_member[i++] = new brick(p);
//	grp_member[i] = NULL;
	}

axisfour::axisfour(param& p)
	{
	int i = 0;
	init(p,5);
	double x1 = sha_org.x() + 2;
	double x2 = sha_org.x() + sha_siz.x() - 2;
	double y1 = sha_org.y();
	double y2 = sha_org.y() + sha_siz.y();
	double z  = sha_org.z() + 2;
	vector ndir(0,-1,0);
	vector norg(x1,y1,z);
	param navepar(norg,ndir,sha_col,sha_rot);
	grp_member[i++] = new navepin1(navepar);
	norg = vector(x2,y1,z);
	navepar.setorg(norg);
	grp_member[i++] = new navepin1(navepar);
	ndir = vector(0,1,0);
	norg = vector(x2,y2,z);
	navepar.setsiz(ndir);
	navepar.setorg(norg);
	grp_member[i++] = new navepin1(navepar);
	norg = vector(x1,y2,z);
	navepar.setsiz(ndir);
	navepar.setorg(norg);
	grp_member[i++] = new navepin1(navepar);
	grp_member[i++] = new brick(p);
//	grp_member[i] = NULL;
	}

axisflat::axisflat(param& p)
	{
	int i = 0;
	init(p,3);
	double x  = sha_org.x() + sha_siz.x()*0.5;
	double y1 = sha_org.y();
	double y2 = sha_org.y() + sha_siz.y();
	double z  = sha_org.z();
	vector ndir(0,-1,0);
	vector norg(x,y1,z);
	param navepar(norg,ndir,sha_col,sha_rot);
	grp_member[i++] = new navepin2(navepar);
	ndir = vector(0,+1,0);
	norg = vector(x,y2,z);
	navepar.setsiz(ndir);
	navepar.setorg(norg);
	grp_member[i++] = new navepin2(navepar);
	grp_member[i++] = new brick(p);
//	grp_member[i] = NULL;
	}

rim::rim(param& p)
	{
	double arc;
	int i = 0;
	init(p,5);
	if(p.extrastr()!="") arc = p.extraint();
	else arc = 360.0*(double)random()/(double)RAND_MAX;
	vector ndir(0,1,0);
	vector norg = sha_org+vector(0,1,0);
	vector norg1 = norg + M_SQRT1_2*circularXZ(arc);
	vector norg2 = norg + M_SQRT1_2*circularXZ(arc+90);
	vector norg3 = norg + M_SQRT1_2*circularXZ(arc+180);
	vector norg4 = norg + M_SQRT1_2*circularXZ(arc+270);
	param studpar(norg1,0.24*ndir.unit(),sha_col,sha_rot);
	grp_member[i++] = new stud1(studpar);
	studpar.setorg(norg2);
	grp_member[i++] = new stud1(studpar);
	studpar.setorg(norg3);
	grp_member[i++] = new stud1(studpar);
	studpar.setorg(norg4);
	grp_member[i++] = new stud1(studpar);
	grp_member[i++] = new rimbody(p);
//	grp_member[i] = NULL;
	}

wheel::wheel(param& p)
	{
	int i = 0;
	init(p,2);
	param tyrepar(sha_org,sha_siz,BLACK,sha_rot);
	grp_member[i++] = new tyre(tyrepar);
	grp_member[i++] = new rim(p);
//	grp_member[i] = NULL;
	}

wheelsmall::wheelsmall(param& p)
	{
	int i = 0;
	init(p,2);
	param tyrepar(sha_org,sha_siz,BLACK,sha_rot);
	grp_member[i++] = new tyre(tyrepar);
	grp_member[i++] = new rimsmall(p);
//	grp_member[i] = NULL;
	}

doorleft::doorleft(param& p)
	{
	int i = 0;
	init(p,2);
	vector rel(0.5,0.5,sha_siz.z());
	grp_member[i++] = new stud1(rel,p);
	grp_member[i++] = new doorleftbody(p);
//	grp_member[i] = NULL;
	}

doorright::doorright(param& p)
	{
	int i = 0;
	init(p,2);
	vector rel(0.5,0.5,sha_siz.z());
	grp_member[i++] = new stud1(rel,p);
	grp_member[i++] = new doorrightbody(p);
//	grp_member[i] = NULL;
	}

turndisk2::turndisk2(param& p)
	{
	int i = 0;
	init(p,6);
	double phi = p.extraflt();
	vector dx(circularXY(phi));
	vector dy(circularXY(phi+90));
	vector org(0.5*sha_siz.x(),0.5*sha_siz.y(),sha_siz.z());
	vector zshift(0,0,0.05);
	vector v(org-0.5*dx-0.5*dy);
	param turnp(p);
	color turncol(p.extrastr());
	if(turncol.good()) turnp.setcol(turncol);
	turnp.shiftorg(zshift);
	grp_member[i++] = new stud1(v      ,turnp);
	grp_member[i++] = new stud1(v+dx   ,turnp);
	grp_member[i++] = new stud1(v+   dy,turnp);
	grp_member[i++] = new stud1(v+dx+dy,turnp);
	grp_member[i++] = new turndiskbody2(turnp);
	grp_member[i++] = new body(p);
//	grp_member[i] = NULL;
	}

turndisk4::turndisk4(param& p)
	{
	int i = 0;
	init(p,14);
	double phi = p.extraflt();
	vector dx(circularXY(phi));
	vector dy(circularXY(phi+90));
	vector org(0.5*sha_siz.x(),0.5*sha_siz.y(),sha_siz.z());
	vector v(org-1.5*dx-0.5*dy);
	grp_member[i++] = new stud1(v,p);
	grp_member[i++] = new stud1(v+dy,p);
	v += dx;
	grp_member[i++] = new stud1(v,p);
	grp_member[i++] = new stud1(v+dy,p);
	v += dx;
	grp_member[i++] = new stud1(v,p);
	grp_member[i++] = new stud1(v+dy,p);
	v += dx;
	grp_member[i++] = new stud1(v,p);
	grp_member[i++] = new stud1(v+dy,p);
	v = org -0.5*dx -1.5*dy;
	grp_member[i++] = new stud1(v,p);
	grp_member[i++] = new stud1(v+dx,p);
	v += 3*dy;
	grp_member[i++] = new stud1(v,p);
	grp_member[i++] = new stud1(v+dx,p);
	param plateparam(p);
	vector platesiz(p.siz().x(),p.siz().y(),1);
	plateparam.setsiz(platesiz);
	grp_member[i++] = new turndiskbody4(p);
	grp_member[i++] = new body(plateparam);
//	grp_member[i] = NULL;
	}

window::window(param& p)
	{
	int i = 0;
	int members = 1 + (int)p.siz().x();
	init(p,members);
	vector v(vector(0.5,0.5*sha_siz.y(),sha_siz.z()));
	int limit = (int)sha_siz.x();
	for(int j=0;j<limit;j++)
		{
		grp_member[i++] = new stud2(v,p);
		v += XUNIT;
		}
	grp_member[i++] = new windowbody(p);
//	grp_member[i] = NULL;
	}

wedgeleft::wedgeleft(param& p)
	{
	int i = 0;
	int members = 1 + (int)p.siz().y();
	init(p,members);
	vector v(vector(p.siz().x()-0.5,0.5,sha_siz.z()));
	int limit = (int)sha_siz.y();
	for(int j=0;j<limit;j++)
		{
		grp_member[i++] = new stud1(v,p);
		v += YUNIT;
		}
	grp_member[i++] = new wedgeleftbody(p);
//	grp_member[i] = NULL;
	}

wedgeright::wedgeright(param& p)
	{
	int i = 0;
	int members = 1 + (int)p.siz().y();
	init(p,members);
	vector v(vector(0.5,0.5,sha_siz.z()));
	int limit = (int)sha_siz.y();
	for(int j=0;j<limit;j++)
		{
		grp_member[i++] = new stud1(v,p);
		v += YUNIT;
		}
	grp_member[i++] = new wedgerightbody(p);
//	grp_member[i] = NULL;
	}

towball::towball(param& p)
	{
	int i = 0;
	int members = 2;
	init(p,members);
	vector orgp(0.5*p.siz().x(),p.siz().y(),0.5*p.siz().z());
	orgp += p.org();
	param pp(orgp,vector(0,0,0),p.col(),p.rot(),p.extrastr());
	grp_member[i++] = new towballpart(pp);
	grp_member[i++] = new brick(p);
//	grp_member[i] = NULL;
	}

towsocket::towsocket(param& p)
	{
	int i = 0;
	int members = 2;
	init(p,members);
	vector orgp(0.5*p.siz().x(),p.siz().y(),0.5*p.siz().z());
	orgp += p.org();
	param pp(orgp,vector(0,0,0),p.col(),p.rot(),p.extrastr());
	grp_member[i++] = new towsocketpart(pp);
	grp_member[i++] = new brick(p);
//	grp_member[i] = NULL;
	}

fence1::fence1(param& p)
	{
	int i = 0;
	int members = 4+(int)p.siz().y();
	init(p,members);
	param pp(p);
	pp.setsiz(1,sha_siz.y(),1);
	grp_member[i++] = new body(pp);
	pp.setsiz(1,sha_siz.y(),0.5);
	vector v(sha_org.x(),sha_org.y(),sha_org.z()+sha_siz.z()-0.5);
	pp.setorg(v);
	grp_member[i++] = new body(pp);
	v = vector(0.5,0.5,sha_siz.z());
	grp_member[i++] = new stud2(v,p);
	v += vector(0,sha_siz.y()-1,0);
	grp_member[i++] = new stud2(v,p);
	param ppp(sha_org+vector(0.5,0.5,1),vector(0,0,sha_siz.z()-1.5),sha_col,sha_rot);
	v = vector(0,1,0);
	for(int j=0;j<sha_siz.y();j++)
		{
		grp_member[i++] = new fence1part(ppp);
		ppp.shiftorg(v);
		}
//	grp_member[i] = NULL;	
	}

fence2::fence2(param& p)
	{
	int i = 0;
	int members = 3 + 3*(int)p.siz().y();
	init(p,members);
	param pbody(p);
	pbody.setsiz(1,sha_siz.y(),1);
	grp_member[i++] = new body(pbody);
	double palethick = 0.25;
	double palewidth = 0.625;
	vector paleorg(sha_org+vector(0.5-0.5*palethick,0.5-0.5*palewidth,1));
	vector palesiz(palethick,palewidth,sha_siz.z()-1);
	param ppale(paleorg,palesiz,sha_col,sha_rot);
	for(int j=0;j<sha_siz.y();j++)
		{	
		grp_member[i++] = new fence2part(ppale);
		ppale.shiftorg(YUNIT);
		}
	double propthick = 0.2;	
	double propheigh = 0.625;
	double propzpos1 = 1.45;
	double propzpos2 = 4.15;
	vector proporg1(sha_org+vector(0.5-0.5*propthick,0,propzpos1));
	vector proporg2(sha_org+vector(0.5-0.5*propthick,0,propzpos2));
	vector propsiz(propthick,(1-palewidth)*0.5,propheigh);
	param pprop1(proporg1,propsiz,sha_col,sha_rot);
	param pprop2(proporg2,propsiz,sha_col,sha_rot);
	grp_member[i++] = new body(pprop1);
	grp_member[i++] = new body(pprop2);
	proporg1 = proporg2 = sha_org;
	proporg1 += vector(0.5-0.5*propthick,sha_siz.y()-(1-palewidth)*0.5,propzpos1);
	proporg2 += vector(0.5-0.5*propthick,sha_siz.y()-(1-palewidth)*0.5,propzpos2);
	pprop1 = param(proporg1,propsiz,sha_col,sha_rot);
	pprop2 = param(proporg2,propsiz,sha_col,sha_rot);
	grp_member[i++] = new body(pprop1);
	grp_member[i++] = new body(pprop2);
	propsiz = vector(propthick,1-palewidth,propheigh);
	proporg1 = proporg2 = sha_org;
	proporg1 += vector(0.5-0.5*propthick,0.5+palewidth*0.5,propzpos1);
	proporg2 += vector(0.5-0.5*propthick,0.5+palewidth*0.5,propzpos2);
	pprop1 = param(proporg1,propsiz,sha_col,sha_rot);
	pprop2 = param(proporg2,propsiz,sha_col,sha_rot);
	for(int j=0;j<sha_siz.y()-1;j++)
		{	
		grp_member[i++] = new body(pprop1);
		grp_member[i++] = new body(pprop2);
		pprop1.shiftorg(YUNIT);
		pprop2.shiftorg(YUNIT);
		}
//	grp_member[i] = NULL;
	}


////////////////////////////////////////////////////////////////////

test::test(param& p)
	{
	int i = 0;
	int members = 1;
	init(p,members);
	grp_member[i++] = new testbody(p);
//	grp_member[i] = NULL;
	}

//
