// lego.cc	v1.3	10. September 2008	by eddy

/*  lego -- this program generates images of LEGO(R)-like models.

    		Copyright (C) 1998 - 2008 Edmund Nussbaum.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that you will have a lot of 
    fun when using it, but WITHOUT ANY WARRANTY; without even the implied 
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
    the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Please report your experiences, suggestions or whatever related
    to this program to: 
                       
    sibylle.eddy @ t-online.de
    en @ mpifr-bonn.mpg.de 

    or send mail to:
    Edmund Nussbaum, Richard-Wagner-Str. 59, 53115 Bonn, Germany 
*/

#include "line.h"
#include "vector.h"
#include "shape.h"
#include "group.h"
#include "room.h"
#include "signatur.h"
#include "param.h"
#include "String.h"
#include <iostream>
#include <sstream>
#include <iomanip>
#include <unistd.h>
#include <stdlib.h>

using namespace std;

void usage(String argv0)
	{
	cerr << "Usage:\n";
	cerr << argv0 << " [hs] < input.lego > output.ppm \n";
	cerr << "option -help	this help screen \n";
	cerr << "option -seq 	create sequence of output files, \n";
	cerr << " 		one image for each processed brick. \n";
	cerr << " 		take care of your storage capacity! \n";
	exit(1);
	}

int main(int argc, char* argv[])
{
cerr << "\n***   l e g o   ***   by eddy   ***   November 2008   ***\n";
room* rp;
int imgcount = 0;
if(argc>1) 
	{
	String arg;
	for(int i=1;i<argc;i++) arg += argv[i];
	if(arg.contains('h')) usage(argv[0]);
	if(arg.contains('s')) imgcount = 1;
	}
srandom(getpid());
while(true)
	{
	line l;
	if(!l.more())
	    {
	    cerr<<"\nError: No Room or Raum statement found !\n";
	    exit(2);
	    }
	if(!l.valid()) continue;
	if(!l.kequal("Raum","Room")) continue;
	vector rsize(l.fltarg(2),l.fltarg(3),l.fltarg(4));
	rp = new room(l.intarg(1),rsize,l.fltarg(5),l.fltarg(6));
	break;
	}
room& r = *rp;
if(imgcount) r.writeppm("img0000.ppm");
vector shiftvec;	// Offset vector.
String sigstr;		// Signatur string.
while(true)
	{
	line l;
	if(l.valid())
	    {
	    if(l.kequal("Nullpunkt","Offset"))
			{
			shiftvec = vector(l.fltarg(1),l.fltarg(2),l.fltarg(3));
			cerr << endl;
			continue;
			}
	    else if(l.kbegin("Signatur"))
			{
			if(sigstr.length()>0) sigstr += '\n';
			sigstr += l.strarg(1);
			for(int i=2;i<l.nargs();i++)
		    	{
		    	sigstr += ' ';
		    	sigstr += l.strarg(i);
		    	}
			cerr << endl;
			continue;
			}
	    else if(l.kbegin("End")) break;

	    param par(l);
	    par.shiftorg(shiftvec);
	    if(l.kbegin("Stein","Block"))
		{
		par.setsiz(l.knum1(),l.knum2(),l.knum3()==0 ? 3 : l.knum3());
		brick s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Flach","Flat"))
		{
		par.setsiz(l.knum1(),l.knum2(),1);
		brick s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Kachel","Tile"))
		{
		par.setsiz(l.knum1(),l.knum2(),1);
		body s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Gitter","Grille"))
		{
		par.setsiz(1,2,1);
		grille s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Grund","Ground"))
		{
		par.setsiz(l.knum1(),l.knum2(),0.35);
		ground s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Dachspitz22","Rooftop22"))
		{
		par.setsiz(l.knum2(),2,1.8);
		rooftop s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Dachspitz45","Rooftop45"))
		{
		par.setsiz(l.knum2(),2,3);
		rooftop s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Dacheck22","Roofcorner22"))
		{
		par.setsiz(3,3,3);
		roofcorner s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Dacheck45","Roofcorner45"))
		{
		par.setsiz(2,2,3);
		roofcorner s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Antidach22","Antiroof22"))
		{
		par.setsiz(l.knum2(),3,3);
		antiroof s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Antidach45","Antiroof45"))
		{
		par.setsiz(l.knum2(),2,3);
		antiroof s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Dach22","Roof22"))
		{
		par.setsiz(l.knum2(),3,3);
		roof s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Dach45","Roof45"))
		{
		par.setsiz(l.knum2(),2,3);
		roof s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Eckeflach","Cornerflat"))
		{
		par.setsiz(2,2,1);
		corner s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Ecke","Corner"))
		{
		par.setsiz(2,2,3);
		corner s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Rundflach","Roundflat"))
		{
		par.setsiz(1,1,1);
		round1 s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Rundhoch","Roundhigh"))
		{
		par.setsiz(1,1,3);
		round1 s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Rundtyp1","Curvedtype1"))
		{
		par.setsiz(2,3,3);
		curved1 s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Rundtyp2","Curvedtype2"))
		{
		par.setsiz(2,4,6);
		curved2 s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Rundtyp3","Curvedtype3"))
		{
		par.setsiz(2,4,6);
		curved3 s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Rundtyp4","Curvedtype4"))
		{
		par.setsiz(2,l.knum2()==0 ? 4 : l.knum2(),3);
		curved4 s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Rundtyp5","Curvedtype5"))
		{
		par.setsiz(2,l.knum2()==0 ? 2 : l.knum2(),3);
		curved5 s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Rundeck1","Roundcorner1"))
		{
		par.setsiz(2,2,3);
		roundcorner1 s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Rundeck2","Roundcorner2"))
		{
		par.setsiz(4,4,3);
		roundcorner2 s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Achsevertflach","Axisvertflat"))
		{
		par.setsiz(2,2,1);
		axisvertical s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Achseverthoch","Axisverthigh"))
		{
		par.setsiz(2,2,3);
		axisvertical s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Achseeins","Axisone"))
		{
		par.setsiz(l.knum1(),l.knum2(),3);
		axisone s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Achsezwei","Axistwo"))
		{
		par.setsiz(l.knum1(),l.knum2(),3);
		axistwo s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Achsevier","Axisfour"))
		{
		par.setsiz(10,4,3);
		axisfour s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Achseflach","Axisflat"))
		{
		par.setsiz(l.knum1()==0 ? 2 : l.knum1(),l.knum2()==0 ? 2 : l.knum2(),1);
		axisflat s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Felgeklein","Rimsmall"))
		{
//		par.setsiz(0,0,0);
		param mypar(par.org(),vector(0,0.75,0),par.col(),
		            par.rot(),par.extrastr());
		rimsmall s(mypar); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Felge","Rim"))
		{
//		par.setsiz(0,0,0);
		param mypar(par.org(),vector(0,1,0),par.col(),
                            par.rot(),par.extrastr());
		rim s(mypar); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Reifenklein","Tyresmall"))
		{
//		par.setsiz(0,0,0);
		param mypar(par.org(),vector(0,0.75,0),par.col(),
		            par.rot(),par.extrastr());
		tyre s(mypar); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Reifen","Tyre"))
		{
//		par.setsiz(0,0,0);
		param mypar(par.org(),vector(0,1,0),par.col(),
		            par.rot(),par.extrastr());
		tyre s(mypar); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Radklein","Wheelsmall"))
		{
//		par.setsiz(0,0,0);
		param mypar(par.org(),vector(0,0.75,0),par.col(),
		            par.rot(),par.extrastr());
		wheelsmall s(mypar); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Rad","Wheel"))
		{
//		par.setsiz(0,0,0);
		param mypar(par.org(),vector(0,1,0),par.col(),
		par.rot(),par.extrastr());
		wheel s(mypar); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Tuerlinks","Doorleft"))
		{
		par.setsiz(1,1,l.knum2());
		param mypar(par.org(),vector(1,l.knum1(),l.knum2()),par.col(),
		            par.rot(),par.extrastr());
		doorleft s(mypar); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Tuerrechts","Doorright"))
		{
		par.setsiz(1,1,l.knum2());
		param mypar(par.org(),vector(1,l.knum1(),l.knum2()),par.col(),
		            par.rot(),par.extrastr());
		doorright s(mypar); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Drehscheibe2","Turndisk2"))
		{
		par.setsiz(2,2,1);
		turndisk2 s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Drehscheibe4","Turndisk4"))
		{
		par.setsiz(4,4,3);
		turndisk4 s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Fenster","Window"))
		{
		par.setsiz(l.knum1(),1,l.knum2()==0 ? 6 : l.knum2());
		window s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Auge","Eye"))
		{
		par.setsiz(1,0,3);
		eye s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Keillinks","Wedgeleft"))
		{
		par.setsiz(2,3,3);
		wedgeleft s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Keilrechts","Wedgeright"))
		{
		par.setsiz(2,3,3);
		wedgeright s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kequal("Henkel","Handle"))
		{
		par.setsiz(2,1,4);
		handle s(par); s.draw(r); cerr << endl;
		}
		else if(l.kbegin("Kupplungball","Towball"))
		{
		par.setsiz(l.knum1(),l.knum2(),l.knum3()==0 ? 1 : l.knum3());
		towball s(par); s.draw(r); cerr << endl;
		}
		else if(l.kbegin("Kupplungloch","Towsocket"))
		{
		par.setsiz(l.knum1(),l.knum2(),l.knum3()==0 ? 1 : l.knum3());
		towsocket s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Zaun1","Fence1"))
		{
		par.setsiz(1,4,6);
		fence1 s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Zaun2","Fence2"))
		{
		par.setsiz(1,4,6);
		fence2 s(par); s.draw(r); cerr << endl;
		}
	    else if(l.kbegin("Test"))
		{
		par.setsiz(1,1,3);
		test s(par); s.draw(r); cerr << endl;
		}
	    else
		{
		cerr << "Warning: Line " << l.linenumber() << ": ";
		cerr << "Bad keyword " << l.keyword() << endl;
		}
		if(imgcount)
			{
			signature sig(sigstr);
			sig.draw(r);
			ostringstream ostr;
			ostr<<"img"<<setfill('0')<<setw(4)<<imgcount++<<".ppm"<<ends;
			r.writeppm(ostr.str().c_str());
			}
	    }
	if(!l.more()) break;
	}
signature sig(sigstr);
sig.draw(r);
r.writeppm();
delete rp;
return(0);
}

//
