// line.cc	v1.3	19. March 2005	by eddy

/*  lego -- this program generates images of LEGO(R)-like models.

    		Copyright (C) 1998 - 2008 Edmund Nussbaum.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that you will have a lot of 
    fun when using it, but WITHOUT ANY WARRANTY; without even the implied 
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
    the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Please report your experiences, suggestions or whatever related
    to this program to: 

    sibylle.eddy @ t-online.de
    en @ mpifr-bonn.mpg.de 

    or send mail to:
    Edmund Nussbaum, Richard-Wagner-Str. 59, 53115 Bonn, Germany 
*/

#include "line.h"
#include <sstream>

using namespace std;

static int lno = 0;	// Line number.

line::line(void)
	{
	moreflag = validflag = true;
	keynumflag = false;
	lineno = ++lno;
	for(int i=0;i<MAXARGS;i++) linearg[i] = "";
	char tmp[MAXLL];
	cin.getline(tmp,MAXLL);
	if( !cin.eof() && cin.fail() )
		{
		cerr << "Warning in line " << lineno;
		cerr << ": Line is too long.\n";
		cin.clear();
		int c;
		do c = cin.get(); while( (c!='\n')&&!cin.eof() );
		}
	if(cin.eof()) moreflag = false;
	numargs = split(tmp,linearg,MAXARGS,RXwhite);
	if(linearg[0]=="")
		{
		for(int i=0;i<MAXARGS-1;i++) linearg[i] = linearg[i+1];
		linearg[MAXARGS-1] = "";
		numargs--;
		}
	if(numargs<=0)              validflag = false;
	else if(linearg[0]=="")     validflag = false;
	else if(linearg[0][0]=='#') validflag = false;
	else for(int i=0;i<MAXARGS;i++) cerr << linearg[i] << ' '; 
	}

String line::keyword(void)    
	{ 
	return capitalize( downcase( linearg[0] ) ); 
	}

String line::strarg(int i)
	{
	if(i>=MAXARGS) return(""); 
	return linearg[i];
	}

int line::intarg(int i)
	{
	istringstream istr( (const char *)(linearg[i]) );
	int val = 0;
	istr >> val;
	return val;
	}

float line::fltarg(int i)
	{
	istringstream istr( (const char *)(linearg[i]) );
	float val = 0;
	istr >> val;
	return val;
	}

bool line::kequal(const String& str)
	{
	if(str==keyword()) return true;
	return false;
	}

bool line::kequal(const String& str1,const String& str2)
	{
	if(str1==keyword()) return true;
	if(str2==keyword()) return true;
	return false;
	}

bool line::kbegin(const String& str)
	{
	if(keyword().contains(str, 0)) return true;
	return false;
	}

bool line::kbegin(const String& str1,const String& str2)
	{
	if(keyword().contains(str1, 0)) return true;
	if(keyword().contains(str2, 0)) return true;
	return false;
	}

void line::parsekeynums(void)
	{
	keynumflag = true;	
	keyn1 = keyn2 = keyn3 = 0;
	String str = keyword();
	if(!str.contains(RXint)) return;
	String keystr1 = str.at(RXint);
	istringstream istr1((const char *)keystr1);
	istr1 >> keyn1;
	str = str.after(keystr1);
	if(!str.contains(RXint)) return;
	String keystr2 = str.at(RXint);
	istringstream istr2((const char *)keystr2);
	istr2 >> keyn2;
	str = str.after(keystr2);
	if(!str.contains(RXint)) return;
	String keystr3 = str.at(RXint);
	istringstream istr3((const char *)keystr3);
	istr3 >> keyn3;
	}

int line::knum1(void)
	{
	if(!keynumflag) parsekeynums();
	return keyn1;	
	}

int line::knum2(void)
	{
	if(!keynumflag) parsekeynums();
	return keyn2;	
	}

int line::knum3(void)
	{
	if(!keynumflag) parsekeynums();
	return keyn3;	
	}

//
