// param.cc	v1.3	9. June 2007	by eddy

/*  lego -- this program generates images of LEGO(R)-like models.

    		Copyright (C) 1998 - 2008 Edmund Nussbaum.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that you will have a lot of 
    fun when using it, but WITHOUT ANY WARRANTY; without even the implied 
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
    the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Please report your experiences, suggestions or whatever related
    to this program to: 

    sibylle.eddy @ t-online.de
    en @ mpifr-bonn.mpg.de 

    or send mail to:
    Edmund Nussbaum, Richard-Wagner-Str. 59, 53115 Bonn, Germany 
*/

#include "param.h"
#include <iostream>
#include <sstream>

using namespace std;

param::param(line& l)
	{
	rotvalidflag = false;
	rotfixedflag = false;
	porg = vector(l.fltarg(1),l.fltarg(2),l.fltarg(3));
	pcol = color(l.strarg(4));
	if(!pcol.good())
		{
		cerr << "Warning in line " << l.linenumber();
		cerr << ": Bad color " << l.strarg(4);
		cerr << ". Use white instead.\n";
		pcol = color("white");
		}
	char rotchar = '\0';
	if(!l.strarg(5).empty()) rotchar = upcase(l.strarg(5))[0];
	protalign = N;
	switch(rotchar)
		{
		case '\0': 
		case 'N' : break;
		case 'S' : protalign = S; break;
		case 'W' : protalign = W; break;
		case 'O' :
		case 'E' : protalign = E; break;
		default  :	{
				cerr << "Warning in line " << l.linenumber();
				cerr << ": Bad alignment " << l.strarg(5);
				cerr << ". Use north instead.\n";
			   	}
		}
	int i=6;
	pextrastr = "";
	while(true)
		{
		String tmpstr = l.strarg(i++);
		if(tmpstr.empty()) break;
		pextrastr += " ";
		pextrastr += tmpstr;
		}
	}

param::param(const vector& org,const vector& siz,color col,
             const rotator& rot,const String extra)
	{
	porg = org;
	psiz = siz;
	pcol = col;
	prot = rot;
	rotvalidflag = true;
	rotfixedflag = true;
	pextrastr = extra;
	}

const rotator& param::rot(void)
	{
	if(!rotvalidflag)
		{
		prot = rotator(porg,psiz,protalign);
		rotvalidflag = true;
		}
	return prot;
	}

void param::setsiz(double x,double y,double z)
	{
	setsiz(vector(x,y,z));
	}

void param::setsiz(const vector& vec)
	{
	if(!rotfixedflag) rotvalidflag = false;
	psiz = vec;
	}

void param::setorg(const vector& vec)
	{
	if(!rotfixedflag) rotvalidflag = false;
	porg = vec;
	}

void param::shiftorg(const vector& v)
	{
	if(!rotfixedflag) rotvalidflag = false;
	porg += v;
	}

void param::setcol(color c)
	{
	pcol = c;
	}

int param::extraint(void)
	{
	int intval = 0;
	istringstream istr((const char *)pextrastr);
	istr >> intval;
	return intval;
	}

double param::extraflt(void)
	{
	double fltval = 0;
	istringstream istr((const char *)pextrastr);
	istr >> fltval;
	return fltval;
	}

String param::extrastr(void) { return pextrastr; }

//
