// param.h	v1.3	9. June 2007	by eddy

/*  lego -- this program generates images of LEGO(R)-like models.

    		Copyright (C) 1998 - 2008 Edmund Nussbaum.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that you will have a lot of 
    fun when using it, but WITHOUT ANY WARRANTY; without even the implied 
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
    the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Please report your experiences, suggestions or whatever related
    to this program to: 
    
    sibylle.eddy @ t-online.de                   
    en @ mpifr-bonn.mpg.de 

    or send mail to:
    Edmund Nussbaum, Richard-Wagner-Str. 59, 53115 Bonn, Germany 
*/

#ifndef _PARAM_H
#define _PARAM_H

#include "global.h"
#include "line.h"
#include "vector.h"
#include "rotator.h"
#include "color.h"
#include "String.h"

class param
	{
	private:
	bool rotvalidflag;	// Indicates validation of prot.
	bool rotfixedflag;	// Indicates that prot is fix.
	vector porg;		// Origin, may be modyfied by param::shiftorg().
	vector psiz;		// Size, can be set by param::setsiz().
	color pcol;			// Color.
	rotator prot;		// Depends on porg, psiz, protdir if rotfixedflag is false.
	dir protalign;		// Defines alignment and rotation.
	String pextrastr;	// Line arg, aditional parameter.
	
	public:
	param(line&);
	param(const vector& org,const vector& siz,color col,
	      const rotator& rot,const String extra="");
	~param(void) { }
	const vector& org(void) const { return porg; }
	const vector& siz(void) const { return psiz; }
	const color&  col(void) const { return pcol; }
	const rotator& rot(void);
	void setsiz(double x,double y,double z);
	void setsiz(const vector&);
	void setorg(const vector&);
	void shiftorg(const vector&);
	void setcol(color c);
	int    extraint(void);		// Returns extra parameter as integer.
	double extraflt(void);		// Returns extra parameter as double.
	String extrastr(void);		// Returns extra parameter as String.
	};

#endif // _PARAM_H
