// shape.h	v1.3	9. June 2007	by eddy

/*  lego -- this program generates images of LEGO(R)-like models.

    		Copyright (C) 1998 - 2008 Edmund Nussbaum.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that you will have a lot of 
    fun when using it, but WITHOUT ANY WARRANTY; without even the implied 
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
    the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Please report your experiences, suggestions or whatever related
    to this program to: 

    sibylle.eddy @ t-online.de
    en @ mpifr-bonn.mpg.de 

    or send mail to:
    Edmund Nussbaum, Richard-Wagner-Str. 59, 53115 Bonn, Germany 
*/

#ifndef _SHAPE_H
#define _SHAPE_H

#include "vector.h"
#include "rotator.h"
#include "room.h"
#include "global.h"
#include "param.h"

class shape			// generic object
	{
	protected:
	vector  sha_org;	// origin
	vector  sha_siz;	// size
	color   sha_col;	// main color
	rotator sha_rot;	// rotation
	public:
	shape(void) { }
	shape(param&);
	virtual ~shape(void) { }
	virtual void draw(room&) { }
	};

class stud1 : public shape	
	{
	public:
	stud1(param& p) : shape(p) { }
	stud1(const vector& rel,param& p);	// Place stud relative to p.
	void draw(room&);
	};

class stud2 : public shape	
	{
	public:
	stud2(param& p) : shape(p) { }
	stud2(const vector& rel,param& p);	// Place stud relative to p.
	void draw(room&);
	};

class body : public shape	// box with outlines
	{
	public:
	body(param& p) : shape(p) { }
	void draw(room&);
	};

class groundbody : public shape
	{
	public:
	groundbody(param& p) : shape(p) { }
	void draw(room&);
	};

class antiroofbody : public shape
	{
	public:
	antiroofbody(param& p) : shape(p) { }
	void draw(room&);
	};

class roofbody : public shape
	{
	public:
	roofbody(param& p) : shape(p) { }
	void draw(room&);
	};

class rooftop : public shape
	{
	public:
	rooftop(param& p) : shape(p) { }
	void draw(room& r);
	};

class roofcornerbody : public shape
	{
	public:
	roofcornerbody(param& p) : shape(p) { }
	void draw(room&);
	};

class cornerbody : public shape
	{
	public:
	cornerbody(param& p) : shape(p) { }
	void draw(room&);
	};

class roundbody : public shape
	{
	public:
	roundbody(param& p) : shape(p) { }
	void draw(room&);
	};

class curved1body : public shape
	{
	public:
	curved1body(param& p) : shape(p) { }
	void draw(room&);
	};

class curved2body : public shape
	{
	public:
	curved2body(param& p) : shape(p) { }
	void draw(room&);
	};

class curved3 : public shape
	{
	public:
	curved3(param& p) : shape(p) { }
	void draw(room& r);
	};

class curved4 : public shape
	{
	public:
	curved4(param& p) : shape(p) { }
	void draw(room& r);
	};

class roundcorner1body : public shape	
	{
	public:
	roundcorner1body(param& p) : shape(p) { }
	void draw(room&);
	};

class roundcorner2body : public shape	
	{
	public:
	roundcorner2body(param& p) : shape(p) { }
	void draw(room&);
	};

class navepin1 : public shape	
	{
	public:
	navepin1(param& p) : shape(p) { }
	void draw(room&);
	};

class navepin2 : public shape	
	{
	public:
	navepin2(param& p) : shape(p) { }
	void draw(room&);
	};

class rimbody : public shape	
	{
	public:
	rimbody(param& p) : shape(p) { }
	void draw(room&);
	};

class rimsmall : public shape	
	{
	private:
	double rimsmall_arc;
	public:
	rimsmall(param& p);
	void draw(room&);
	};

class tyre : public shape	
	{
	public:
	tyre(param& p) : shape(p) { }
	void draw(room&);
	};

class doorleftbody : public shape	
	{
	public:
	doorleftbody(param& p) : shape(p) { }
	void draw(room&);
	};

class doorrightbody : public shape	
	{
	public:
	doorrightbody(param& p) : shape(p) { }
	void draw(room&);
	};

class turndiskbody2 : public shape	
	{
	public:
	turndiskbody2(param& p) : shape(p) { }
	void draw(room&);
	};

class turndiskbody4 : public shape	
	{
	public:
	turndiskbody4(param& p) : shape(p) { }
	void draw(room&);
	};

class windowbody : public shape	
	{
	public:
	windowbody(param& p) : shape(p) { }
	void draw(room&);
	};

class eye : public shape	
	{
	public:
	eye(param& p) : shape(p) { }
	void draw(room&);
	};

class wedgeleftbody : public shape	
	{
	public:
	wedgeleftbody(param& p) : shape(p) { }
	void draw(room&);
	};

class wedgerightbody : public shape	
	{
	public:
	wedgerightbody(param& p) : shape(p) { }
	void draw(room&);
	};

class grille : public shape
	{
	public:
	grille(param& p) : shape(p) { }
	void draw(room&);
	};

class handle : public shape
	{
	public:
	handle(param& p) : shape(p) { }
	void draw(room&);
	};

class towballpart : public shape
	{
	public:
	towballpart(param& p) : shape(p) { }
	void draw(room&);
	};
	
class towsocketpart : public shape
	{
	public:
	towsocketpart(param& p) : shape(p) { }
	void draw(room&);
	};

class fence1part : public shape	
	{
	public:
	fence1part(param& p) : shape(p) { }
	void draw(room&);
	};

class fence2part : public shape	
	{
	public:
	fence2part(param& p) : shape(p) { }
	void draw(room&);
	};

class testbody : public shape
	{
	public:
	testbody(param& p) : shape(p) { }
	void draw(room&);
	};
	
#endif // _SHAPE_H
