// signatur.cc	v1.3	9. April 2000	by eddy

/*  lego -- this program generates images of LEGO(R)-like models.

    		Copyright (C) 1998 - 2008  by Edmund Nussbaum.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that you will have a lot of 
    fun when using it, but WITHOUT ANY WARRANTY; without even the implied 
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
    the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Please report your experiences, suggestions or whatever related
    to this program to: 
                       
    sibylle.eddy @ t-online.de
    en@mpifr-bonn.mpg.de 

    or send mail to:
    Edmund Nussbaum, Richard-Wagner-Str. 59, 53115 Bonn, Germany 
*/

#include "signatur.h"

signature::signature(const String& str)
	{
	sig_text = str;
	sig_cirdir = vector(0,0,1);
	sig_unit = sig_xchars = sig_ychars = 0;
	sig_ychars = split(sig_text,sig_line,MAXLINES,'\n');
	int i = 0;
	while(sig_line[i]!="")
		{
		int l = sig_line[i].length();
		if(l>sig_xchars) sig_xchars = l;
		i++;
		}
	}

void signature::draw(room& r)
	{
	sig_room = &r;
	sig_unit = r.room_xsize>r.room_ysize ? r.room_xsize : r.room_ysize;
	sig_unit *= sig_scale;
	vector yorg(r.room_xsize,0,r.room_zsize);
	yorg += vector(-(3*sig_xchars+1)*sig_unit,6*sig_unit,0);
	vector xorg(yorg);
	sig_xunit = vector(sig_unit,0,0);
	sig_yunit = vector(0,sig_unit,0);
	for(int y=sig_ychars-1;y>=0;y--)
		{
		for(unsigned x=0;x<sig_line[y].length();x++)
			{
			setchar(xorg,sig_line[y][x]);
			xorg += sig_xunit*3;
			}
		yorg += sig_yunit*6; xorg = yorg;
		}
	}

void signature::setchar(vector pos,char c)
	{
	vector& o = pos;
	vector& d = sig_cirdir;
	vector& x = sig_xunit;
	vector& y = sig_yunit;
	float&  u = sig_unit;
	room&   r = *sig_room;
	color   col = OUTLINE;
	switch(c)
		{
		case ' ': break;
		case 'a': r.docircle(o,d,u,col);
			  r.doline(o+x-y,o+x+y,col);
			  break;
		case 'b': r.docircle(o,d,u,col);
			  r.doline(o-x-y,o-x+3*y,col);
			  break;
		case 'c': r.doarc(o,d,sqrt(0.5)*(x+y),270,col);
			  break;
		case 'd': r.docircle(o,d,u,col);
			  r.doline(o+x-y,o+x+3*y,col);
			  break;
		case 'e': r.doarc(o,d,x,270,col);
			  r.doline(o-x,o+x,col);
			  r.doline(o-y,o+x-y,col);
			  break;
		case 'f': r.doline(o-y,o+2.5*y,col);
			  r.doarc(o+0.5*x+2.5*y,d,0.5*x,180,col);
			  r.doline(o-x+y,o+x+y,col);
			  break;
		case 'g': r.docircle(o,d,u,col);
			  r.doline(o+x+y,o+x-y,col);
			  r.doarc(o-y,d,x,-150,col);
			  break;
		case 'h': r.doline(o-x-y,o-x+3*y,col);
			  r.doarc(o,d,x,180,col);
			  r.doline(o+x-y,o+x,col);
			  break;
		case 'i': r.doline(o-y,o+y,col);
			  r.doline(o+1.9*y,o+2.1*y,col);
			  r.doline(o-0.1*x+2*y,o+0.1*x+2*y,col);
			  break;
		case 'j': r.doline(o-1.5*y,o+y,col);
			  r.doline(o+1.9*y,o+2.1*y,col);
			  r.doline(o-0.1*x+2*y,o+0.1*x+2*y,col);
			  r.doarc(o-0.5*x-1.5*y,d,0.5*x,-180,col);
			  break;
		case 'k': r.doline(o-x-y,o-x+3*y,col);
			  r.doline(o-x,o+x-y,col);
			  r.doline(o-x,o+x+y,col);
			  break;
		case 'l': r.doline(o-0.5*y,o+3*y,col);
			  r.doarc(o+0.5*x-0.5*y,d,-0.5*x,150,col);
			  break;
		case 'm': r.doline(o-x-y,o-x+y,col);
			  r.doarc(o-0.5*x+0.5*y,d,0.5*x,180,col);
			  r.doline(o-y,o+0.5*y,col);
			  r.doarc(o+0.5*x+0.5*y,d,0.5*x,180,col);
			  r.doline(o+x-y,o+x+0.5*y,col);
			  break;
		case 'n': r.doline(o-x-y,o-x+y,col);
			  r.doarc(o,d,x,180,col);
			  r.doline(o+x-y,o+x,col);
			  break;
		case 'o': r.docircle(o,d,u,col);
			  break;
		case 'p': r.docircle(o,d,u,col);
			  r.doline(o-x+y,o-x-2*y,col);
			  break;
		case 'q': r.docircle(o,d,u,col);
			  r.doline(o+x+y,o+x-2*y,col);
			  break;
		case 'r': r.doline(o-0.5*x-y,o-0.5*x+y,col);
			  r.doarc(o+0.5*x ,d,-x,-120,col);
			  break;
		case 's': r.doarc(o+0.5*x+0.5*y,d,0.5*x,90,col);
			  r.doline(o-0.5*x+y,o+0.5*x+y,col);
			  r.doarc(o-0.5*x+0.5*y,d,0.5*y,180,col);
			  r.doline(o-0.5*x,o+0.5*x,col);
			  r.doarc(o+0.5*x-0.5*y,d,0.5*y,-180,col);
			  r.doline(o-0.5*x-y,o+0.5*x-y,col);
			  r.doarc(o-0.5*x-0.5*y,d,-0.5*y,-90,col);
			  break;
		case 't': r.doline(o-0.5*y,o+3*y,col);
			  r.doarc(o+0.5*x-0.5*y,d,-0.5*x,120,col);
			  r.doline(o-x+1*y,o+x+1*y,col);
			  break;
		case 'u': r.doline(o-x,o-x+y,col);
			  r.doarc(o,d,x,-180,col);
			  r.doline(o+x-y,o+x+y,col);
			  break;
		case 'v': r.doline(o-x+y,o-y,col);
			  r.doline(o+x+y,o-y,col);
			  break;
		case 'w': r.doline(o-0.75*x-y,o-x+y,col);
			  r.doline(o-0.75*x-y,o+0.5*y,col);
			  r.doline(o+0.75*x-y,o+0.5*y,col);
			  r.doline(o+0.75*x-y,o+x+y,col);
			  break;
		case 'x': r.doline(o-x-y,o+x+y,col);
			  r.doline(o+x-y,o-x+y,col);
			  break;
		case 'y': r.doline(o-y,o-x+y,col);
			  r.doline(o+x+y,o-0.5*x-2*y,col);
			  break;
		case 'z': r.doline(o-x+y,o+x+y,col);
			  r.doline(o+x+y,o-x-y,col);
			  r.doline(o-x-y,o+x-y,col);
			  break;
		case 'A': r.doline(o-x-y,o+3*y,col);
			  r.doline(o+x-y,o+3*y,col);
			  r.doline(o-0.5*x,o+0.5*x,col);
			  break;
		case 'B': r.doline(o-x-y,o-x+3*y,col);
			  r.doline(o-x-y,o-y,col);
			  r.doline(o-x+y,o+y,col);
			  r.doline(o-x+3*y,o+3*y,col);
			  r.doarc(o+2*y,d,-y,180,col);
			  r.doarc(o,d,y,-180,col);
			  break;
		case 'C': r.doline(o-x,o-x+2*y,col);
			  r.doarc(o+2*y,d,x,180,col);
			  r.doarc(o,d,-x,180,col);
			  break;
		case 'D': r.doline(o-x-y,o-x+3*y,col);
			  r.doarc(o+2*y,d,x,90,col);
			  r.doarc(o,d,x,-90,col);
			  r.doline(o-x+3*y,o+3*y,col);
			  r.doline(o-x-y,o-y,col);
			  r.doline(o+x,o+x+2*y,col);
			  break;
		case 'E': r.doline(o-x-y,o-x+3*y,col);
			  r.doline(o-x-y,o+x-y,col);
			  r.doline(o-x+y,o+0.5*x+y,col);
			  r.doline(o-x+3*y,o+x+3*y,col);
			  break;
		case 'F': r.doline(o-x-y,o-x+3*y,col);
			  r.doline(o-x+y,o+0.5*x+y,col);
			  r.doline(o-x+3*y,o+x+3*y,col);
			  break;
		case 'G': r.doline(o-x,o-x+2*y,col);
			  r.doarc(o+2*y,d,x,180,col);
			  r.doarc(o,d,-x,180,col);
			  r.doline(o+x,o+x+0.5*y,col);
			  r.doline(o+x+0.5*y,o+0.5*y,col);
			  break;
		case 'H': r.doline(o-x-y,o-x+3*y,col);
			  r.doline(o+x-y,o+x+3*y,col);
			  r.doline(o-x+y,o+x+y,col);
			  break;
		case 'I': r.doline(o-y,o+3*y,col);
			  r.doline(o-0.5*x-y,o+0.5*x-y,col);
			  r.doline(o-0.5*x+3*y,o+0.5*x+3*y,col);
			  break;
		case 'J': r.doline(o+x,o+x+3*y,col);
			  r.doarc(o,d,-x,180,col);
			  break;
		case 'K': r.doline(o-x-y,o-x+3*y,col);
			  r.doline(o-x+y,o+x-y,col);
			  r.doline(o-x+y,o+x+3*y,col);
			  break;
		case 'L': r.doline(o-x-y,o-x+3*y,col);
			  r.doline(o-x-y,o+x-y,col);
			  break;
		case 'M': r.doline(o-x-y,o-x+3*y,col);
			  r.doline(o+x-y,o+x+3*y,col);
			  r.doline(o-x+3*y,o+0.5*y,col);
			  r.doline(o+x+3*y,o+0.5*y,col);
			  break;
		case 'N': r.doline(o-x-y,o-x+3*y,col);
			  r.doline(o+x-y,o+x+3*y,col);
			  r.doline(o-x+3*y,o+x-y,col);
			  break;
		case 'O': r.doline(o-x,o-x+2*y,col);
			  r.doline(o+x,o+x+2*y,col);
			  r.doarc(o+2*y,d,x,180,col);
			  r.doarc(o,d,-x,180,col);
			  break;
		case 'P': r.doline(o-x-y,o-x+3*y,col);
			  r.doline(o-x+y,o+y,col);
			  r.doline(o-x+3*y,o+3*y,col);
			  r.doarc(o+2*y,d,-y,180,col);
			  break;
		case 'Q': r.doline(o-x,o-x+2*y,col);
			  r.doline(o+x,o+x+2*y,col);
			  r.doarc(o+2*y,d,x,180,col);
			  r.doarc(o,d,-x,180,col);
			  r.doline(o+0.25*y,o+x-y,col);
			  break;
		case 'R': r.doline(o-x-y,o-x+3*y,col);
			  r.doline(o-x+y,o+y,col);
			  r.doline(o-x+3*y,o+3*y,col);
			  r.doarc(o+2*y,d,-y,180,col);
			  r.doline(o+y,o+x-y,col);
			  break;
		case 'S': r.doarc(o+2*y,d,-y,-270,col);
			  r.doarc(o,d,y,-270,col);
			  break;
		case 'T': r.doline(o-y,o+3*y,col);
			  r.doline(o-x+3*y,o+x+3*y,col);
			  break;
		case 'U': r.doline(o-x,o-x+3*y,col);
			  r.doline(o+x,o+x+3*y,col);
			  r.doarc(o,d,-x,180,col);
			  break;
		case 'V': r.doline(o-y,o-x+3*y,col);
			  r.doline(o-y,o+x+3*y,col);
			  break;
		case 'W': r.doline(o-0.75*x-y,o-x+3*y,col);
			  r.doline(o-0.75*x-y,o+y,col);
			  r.doline(o+0.75*x-y,o+y,col);
			  r.doline(o+0.75*x-y,o+x+3*y,col);
			  break;
		case 'X': r.doline(o-x-y,o+x+3*y,col);
			  r.doline(o+x-y,o-x+3*y,col);
			  break;
		case 'Y': r.doline(o+y,o-x+3*y,col);
			  r.doline(o+y,o+x+3*y,col);
			  r.doline(o+y,o-y,col);
			  break;
		case 'Z': r.doline(o-x+3*y,o+x+3*y,col);
			  r.doline(o+x+3*y,o-x-y,col);
			  r.doline(o-x-y,o+x-y,col);
			  break;
		case '+': r.doline(o-x+y,o+x+y,col);
			  r.doline(o,o+2*y,col);
			  break;
		case '-': r.doline(o-x+y,o+x+y,col);
			  break;
		case '': r.docircle(o+2.5*y,d,0.5*u,col);
			  break;
		case '/': r.doline(o-x-y,o+x+3*y,col);
			  break;
		case '0': r.doarc(o+2*y,d,x,180,col);
			  r.doarc(o,d,-x,180,col);
			  r.doline(o-x,o-x+2*y,col);
			  r.doline(o+x,o+x+2*y,col);
			  break;
		case '1': r.doline(o-x-y,o+x-y,col);
			  r.doline(o-y,o+3*y,col);
			  r.doline(o+3*y,o-x+y,col);
			  break;
		case '2': r.doarc(o+2*y,d,x,180,col);
			  r.doline(o-x-y,o+x+2*y,col);
			  r.doline(o-x-y,o+x-y,col);
			  break;
		case '3': r.doarc(o+2*y,d,-y,270,col);
			  r.doarc(o,d,-x,270,col);
			  break;
		case '4': r.doline(o+0.25*x+3*y,o-x,col);
			  r.doline(o-x,o+x,col);
			  r.doline(o+0.25*x+y,o+0.25*x-y,col);
			  break;
		case '5': r.doarc(o,d,-x,270,col);
			  r.doline(o+y,o-x+y,col);
			  r.doline(o-x+y,o-x+3*y,col);
			  r.doline(o-x+3*y,o+x+3*y,col);
			  break;
		case '6': r.docircle(o,d,u,col);
			  r.doarc(o+2*y,d,x,180,col);
			  r.doline(o-x,o-x+2*y,col);
			  break;
		case '7': r.doline(o-x-y,o+x+3*y,col);
			  r.doline(o-x+3*y,o+x+3*y,col);
			  r.doline(o-x+3*y,o-x+2.5*y,col);
			  r.doline(o-0.5*x+y,o+0.5*x+y,col);
			  break;
		case '8': r.docircle(o+2*y,d,u,col);
			  r.docircle(o,d,u,col);
			  break;
		case '9': r.docircle(o+2*y,d,u,col);
			  r.doarc(o,d,-x,180,col);
			  r.doline(o+x,o+x+2*y,col);
			  break;
		default : r.doline(o-x-2*y,o+x+3*y,RED);
			  r.doline(o-x+3*y,o+x-2*y,RED);
			  r.doline(o-x-2*y,o+x-2*y,BLUE);
			  r.doline(o-x-y,o+x-y,BLUE);
			  r.doline(o-x+y,o+x+y,BLUE);
			  r.doline(o-x+3*y,o+x+3*y,BLUE);
			  r.doline(o-x-2*y,o-x+3*y,YELLOW);
			  r.doline(o+x-2*y,o+x+3*y,YELLOW);
			  r.docircle(o,d,u,GREEN);
		}
	}

void signature::drawcorner(room& r)
	{
	sig_room = &r;
	vector o(0,0,r.room_zsize);
	vector x(r.room_xsize-1,0,r.room_zsize-1);
	vector y(0,r.room_ysize-1,r.room_zsize-1);
	r.doline(o,x,OUTLINE);
	r.doline(x,x+y,OUTLINE);
	r.doline(x+y,y,OUTLINE);
	r.doline(y,o,OUTLINE);
	}

//
